/* pl.rmalinowski.gwtreflect.client.reflect.Class.java 
 Copyright (C) 1998, 1999, 2001, 2005  Free Software Foundation, Inc.

 This file is part of GNU Classpath.

 GNU Classpath is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 GNU Classpath is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
 Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 02110-1301 USA.

 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
 conditions of the GNU General Public License cover the whole
 combination.

 As a special exception, the copyright holders of this library give you
 permission to link this library with independent modules to produce an
 executable, regardless of the license terms of these independent
 modules, and to copy and distribute the resulting executable under
 terms of your choice, provided that you also meet, for each linked
 independent module, the terms and conditions of the license of that
 module.  An independent module is a module which is not derived from
 or based on this library.  If you modify this library, you may extend
 this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */

package pl.rmalinowski.gwtreflect.client.reflect;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import pl.rmalinowski.gwtreflect.client.utils.CollectionUtil;

import com.google.gwt.core.client.GWT;

/**
 * 
 * @author Rafal M.Malinowski (rafal.m.malinowski@gmail.com)
 * @gwtr.progress 60%
 */
public abstract class Class implements Member {

	/**
	 * 
	 * @param className
	 * @return
	 * @throws ClassNotFoundException
	 * @gwtr.progress 40%
	 */
	public static Class forName(String className) throws ClassNotFoundException {
		try {
			return GWTRUtil.getClass(className);
		} catch (GWTRClassNotFoundException ex) {
			throw new ClassNotFoundException(ex.getMessage());
		}
	}

	public static Class getClass(java.lang.Class clazz) throws ClassNotFoundException {
		try {
			return GWTRUtil.getClass(clazz);
		} catch (GWTRClassNotFoundException ex) {
			throw new ClassNotFoundException(ex.getMessage());
		}
	}

	public static Class getClass(String className) throws ClassNotFoundException {
		try {
			return GWTRUtil.getClass(className);
		} catch (GWTRClassNotFoundException ex) {
			throw new ClassNotFoundException(ex.getMessage());
		}
	}

	// TODO change to private
	/**
	 * Contains information about inner classes
	 * 
	 * @gwt.typeArgs <java.lang.String,pl.rmalinowski.gwtreflect.client.reflect.Field>
	 */
	private Map classes = new HashMap();

	/**
	 * Class name ie: my.package.SomeClass
	 */
	protected final String className;

	/**
	 * Contains information about constructors declared in class
	 * 
	 * @gwt.typeArgs <java.lang.String,pl.rmalinowski.gwtreflect.client.reflect.Constructor>
	 */
	protected Map constructors = new HashMap();

	private Class declaringClass;

	/**
	 * Contains information about fields declared in class
	 * 
	 * @gwt.typeArgs <java.lang.String,pl.rmalinowski.gwtreflect.client.reflect.Field>
	 */
	protected Map fields = new HashMap();

	/**
	 * Contains information about methods declared in class
	 * 
	 * @gwt.typeArgs <java.lang.String,pl.rmalinowski.gwtreflect.client.reflect.Method>
	 */
	protected Map methods = new HashMap();

	private int modifiers;

	/**
	 * Class signature Lmy/package/SomeClass
	 */
	protected final String signature;

	/**
	 * Super class (parent class)
	 */
	protected Class superClass; // lazy loaded

	/**
	 * 
	 * Super class name (parent class name)
	 * 
	 */

	protected String superClassName;

	// protected Class(java.lang.Class clazz) {
	/*private Class() {
		// this.className = getClazz().toString().substring(6);
		String _className = "";
		String _signature = "";

		try {
			_className = GWTRUtil.getName(getClazz());

			// GWTRUtil.classes.put(_className, this);
			// signature = "L"+this.className.replace('.', '/')+";";
			_signature = GWTRUtil.toSignature(_className);
			GWT.log("ClassName " + _className + " Signature: " + _signature, null);
		} catch (IllegalAccessException ex) {

		} finally {
			className = _className;
			signature = _signature;
		}
		// TODO :

		// this.fields = fields;
	}*/

	// public abstract Field[] initFields();

	protected Class(String signature,int modifiers) {
		this.signature = signature;
		this.className = GWTRUtil.toClassName(signature);
		this.modifiers = modifiers;
		GWTRUtil.addClass(signature, this);	

	}

	/**
	 * 
	 * @param method
	 * @return
	 */
	protected Object addMethod(Method method) {
		return methods.put(method.getSignaturedName(), method);
	}

	/**
	 * Returns an array containing Class objects representing all the public
	 * classes and interfaces that are members of the class represented by this
	 * Class object.
	 * 
	 * @return Class[]
	 */
	public Class[] getClasses() {
		return CollectionUtil.toArray(classes.values(), new Class[classes.size()]);
		//return (Class[]) classes.values().toArray(new Class[classes.size()]);
	}
	
	/**
	 * ads inner class or interface
	 * @param 
	 * @return
	 */
	protected Object addClass(Class clazz) {
		return classes.put(clazz.getSignature(), clazz);
	}

	/**
	 * 
	 * @return
	 */
	public String getClassName() {
		return className;
	}

	/**
	 * 
	 * @return
	 */
	protected abstract java.lang.Class getClazz();

	/**
	 * @gwtr.progress 0%
	 * @return
	 */
	public Class getComponentType() {
		return null;
	}

	/**
	 * 
	 * @param parameterTypes
	 * @return
	 */
	public Constructor getConstructor(Class[] parameterTypes) throws NoSuchMethodException {
		return null;
	}

	/**
	 * 
	 * @param parameterSignatures
	 * @return
	 */
	public Constructor getConstructor(String[] parameterSignatures) {
		return null;
	}

	/**
	 * 
	 * @return
	 */
	public Constructor[] getConstructors() {
		Object[] o = constructors.values().toArray();
		Constructor[] c = new Constructor[o.length];
		for (int i = 0; i < c.length; i++) {
			c[i] = (Constructor) o[i];
		}
		return c;
	}

	/**
	 * @gwtr.progress 0%
	 * @return
	 */
	public Class[] getDeclaredClasses() {
		return CollectionUtil.toArray(classes.values(), new Class[classes.size()]);
		
	}

	/**
	 * @gwtr.progress 0%
	 * @return
	 */
	public Constructor[] getDeclaredConstructors() {
		return null;
	}

	/**
	 * @gwtr.progress 0%
	 * @return
	 */
	public Field[] getDeclaredFields() {
		return CollectionUtil.toArray(fields.values(), new Field[fields.size()]);
	}

	/**
	 * @gwtr.progress 0%
	 * @param methodName
	 * @param parameterTypes
	 * @return
	 * @throws NoSuchMethodException
	 */
	public Method getDeclaredMethod(String methodName) throws NoSuchMethodException {
		return null;
	}

	/**
	 * @gwtr.progress 0%
	 * @param methodName
	 * @param parameterTypes
	 * @return
	 * @throws NoSuchMethodException
	 */
	public Method getDeclaredMethod(String methodName, Class[] parameterTypes) throws NoSuchMethodException {
		return null;
	}

	/**
	 * @gwtr.progress 0%
	 * @param methodName
	 * @param parameterTypes
	 * @return
	 * @throws NoSuchMethodException
	 */
	public Method getDeclaredMethod(String methodName, java.lang.Class[] parameterTypes) throws NoSuchMethodException {
		return null;
	}

	/**
	 * @gwtr.progress 0%
	 * @param methodName
	 * @param parameterTypes
	 * @return
	 * @throws NoSuchMethodException
	 */
	public Method[] getDeclaredMethods() {
		return CollectionUtil.toArray(methods.values(), new Method[methods.size()]);
	}

	/**
	 * Returns the Class object representing the class or interface that
	 * declares the member or constructor represented by this Member.
	 * 
	 * @gwtr.progress 0%
	 * @return an object representing the declaring class of the underlying
	 *         member
	 */
	public Class getDeclaringClass() {
		return declaringClass;
	}

	/**
	 * 
	 * @param fieldName
	 * @return
	 */
	public Field getField(String fieldName) throws NoSuchFieldException {
		Field _field = (Field) fields.get(fieldName);
		if (_field == null) {
			throw new NoSuchFieldException(fieldName);
		}
		return _field;
	}

	/**
	 * 
	 * @return
	 */
	public Field[] getFields() {
		ArrayList _fields = new ArrayList();
		Iterator iter = fields.values().iterator();
		while (iter.hasNext()) {
			Field _field = (Field) iter.next();
			if (Modifier.isPublic(_field.getModifiers())) _fields.add(_field);
		}
		return CollectionUtil.toArray(_fields, new Field[_fields.size()]);
		// return (Field[]) fields.values().toArray(new Field[fields.size()]);
	}

	/**
	 * @gwtr.progress 0%
	 * @param methodName
	 * @param parameterTypes
	 * @return
	 * @throws NoSuchMethodException
	 */
	public Method getMethod(String methodName) throws NoSuchMethodException {
		Method _method = (Method) methods.get(methodName);
		if (_method == null) {
			throw new NoSuchMethodException(methodName);
		}
		return _method;
	}

	/**
	 * @gwtr.progress 0%
	 * @param methodName
	 * @param parameterTypes
	 * @return
	 * @throws NoSuchMethodException
	 */
	public Method getMethod(String methodName, Class[] parameterTypes) throws NoSuchMethodException {
		// TODO : search for methods in super class

		Method method = (Method) methods.get(GWTRUtil.toMethodSignaturedName(methodName, GWTRUtil.toSignatureString(parameterTypes)));
		if (null == method) {
			throw new NoSuchMethodException();
		}
		return method;
		// return (Method) methods.get(methodName);
	}

	/**
	 * 
	 * @param methodName
	 * @param parameterTypes
	 * @return
	 * @throws NoSuchMethodException
	 */
	public Method getMethod(String methodName, java.lang.Class[] parameterTypes) throws NoSuchMethodException {
		// TODO : search for methods in super class

		Method method = (Method) methods.get(GWTRUtil.toMethodSignaturedName(methodName, GWTRUtil.toSignatureString(parameterTypes)));
		if (null == method)
			throw new NoSuchMethodException();
		return method;
		// return (Method) methods.get(methodName);
	}

	/**
	 * 
	 * @param methodName
	 * @param parameterSignatures
	 * @return
	 */
	public Method getMethod(String methodName, String[] parameterSignatures) {
		// TODO : search for methods in super class
		return (Method) methods.get(methodName);
	}

	/**
	 * 
	 * @return
	 */
	public Method[] getMethods() {
		ArrayList _methods = new ArrayList();
		Iterator iter = methods.values().iterator();
		while (iter.hasNext()) {
			Method _method = (Method) iter.next();
			if (Modifier.isPublic(_method.getModifiers())) _methods.add(_method);
		}
		return CollectionUtil.toArray(_methods, new Method[_methods.size()]);
		//return (Method[]) methods.values().toArray();
	}

	/**
	 * Returns the Java language modifiers for the method represented by this
	 * Method object, as an integer.
	 * 
	 * @gwtr.progress 0%
	 * @return the Java language modifiers for the underlying member
	 */
	public int getModifiers() {
		return modifiers;
	}

	/**
	 * 
	 */
	public String getName() {
		return className;
	}

	/**
	 * 
	 * @return
	 */
	String getSignature() {
		return signature;
	}

	/**
	 * @gwtr.progress 0%
	 * @return
	 */
	public Object[] getSigners() {
		return null;
	}

	/**
	 * @gwtr.progress 0%
	 * @return
	 */
	public String getSimpleName() {
		return null;
	}

	/**
	 * 
	 * @return
	 */
	public Class getSuperclass() {
		return GWTRUtil.toType(getSuperClassSignature());
	}

	/**
	 * 
	 * @return
	 */
	protected String getSuperClassSignature() {
		return "Ljava/lang/Object;";
	}

	/**
	 * 
	 * @return
	 */
	public boolean isArray() {
		throw new UnsupportedOperationException("Array are not implemented yet");
	}

	/**
	 * @gwtr.progress 0%
	 * @return
	 */
	public boolean isInstance(Object o) {
		return false;
	}

	/**
	 * @gwtr.progress 0%
	 * @return
	 */
	public boolean isInterface() {
		return false;
	}

	/**
	 * @gwtr.progress 0%
	 * @return
	 */
	public boolean isPrimitive() {
		return false;
	}

	/**
	 * Create new instance of class
	 * 
	 * @return new instance of class
	 */
	public abstract Object newInstance() throws InstantiationException, IllegalAccessException;

}
