/* pl.rmalinowski.gwtreflect.client.reflect.Constructor.java 
 Copyright (C) 1998, 1999, 2001, 2005  Free Software Foundation, Inc.

 This file is part of GNU Classpath.

 GNU Classpath is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 GNU Classpath is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
 Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 02110-1301 USA.

 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
 conditions of the GNU General Public License cover the whole
 combination.

 As a special exception, the copyright holders of this library give you
 permission to link this library with independent modules to produce an
 executable, regardless of the license terms of these independent
 modules, and to copy and distribute the resulting executable under
 terms of your choice, provided that you also meet, for each linked
 independent module, the terms and conditions of the license of that
 module.  An independent module is a module which is not derived from
 or based on this library.  If you modify this library, you may extend
 this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */

package pl.rmalinowski.gwtreflect.client.reflect;

public abstract class Constructor extends AccessibleObject implements Member {

	private final Class declaringClass;

	private final String name;

	private final int modifiers;

	private final String[] parameterSignatures;

	private final String[] exceptionSignatures;

	// Lazy init
	private Class[] exceptionTypes;

	private Class[] parameterTypes;

	public Constructor(final Class declaringClass, final String name,
			final int modifiers, final String[] parameterSignatures) {
		this(declaringClass, name, modifiers, parameterSignatures,
				new String[] {});
	}

	public Constructor(final Class declaringClass, final String name,
			final int modifiers, final String[] parameterSignatures,
			final String[] exceptionSignatures) {
		super();
		this.declaringClass = declaringClass;
		this.parameterSignatures = parameterSignatures;
		this.modifiers = modifiers;
		this.name = name;
		this.exceptionSignatures = exceptionSignatures;
	}

	public abstract Object newInstance(Object[] initargs)
			throws InstantiationException, IllegalAccessException,
			IllegalArgumentException, InvocationTargetException;

	// {
	// throw new UnsupportedOperationException("Not implemented jet");
	// }

	public Class getDeclaringClass() {
		return declaringClass;
	}

	public int getModifiers() {
		return modifiers;
	}

	public String getName() {
		return name;
	}

	public Class[] getExceptionTypes() {
		return (null == exceptionTypes) ? exceptionTypes = GWTRUtil
				.toTypes(exceptionSignatures) : exceptionTypes;
	}

	public Class[] getParameterTypes() {
		return (null == parameterTypes) ? parameterTypes = GWTRUtil
				.toTypes(parameterSignatures) : parameterTypes;
	}
}
