/* pl.rmalinowski.gwtreflect.client.reflect.Field.java 
 Copyright (C) 1998, 1999, 2001, 2005  Free Software Foundation, Inc.

 This file is part of GNU Classpath.

 GNU Classpath is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 GNU Classpath is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
 Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 02110-1301 USA.

 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
 conditions of the GNU General Public License cover the whole
 combination.

 As a special exception, the copyright holders of this library give you
 permission to link this library with independent modules to produce an
 executable, regardless of the license terms of these independent
 modules, and to copy and distribute the resulting executable under
 terms of your choice, provided that you also meet, for each linked
 independent module, the terms and conditions of the license of that
 module.  An independent module is a module which is not derived from
 or based on this library.  If you modify this library, you may extend
 this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */

package pl.rmalinowski.gwtreflect.client.reflect;

import pl.rmalinowski.gwtreflect.client.utils.NativSignatures;

public abstract class Field extends AccessibleObject implements Member {

    private final Class declaringClass;

    private final String name;

    private final String typeSignature;

    private final int modifiers;

    protected abstract void nativeSet(Object obj, Object value);

    protected abstract Object nativeGet(Object obj);

    public Field(Class declaringClass, String name, int modifiers,
            String typeSignature) {
        this.declaringClass = declaringClass;
        this.name = name;
        this.modifiers = modifiers;
        this.typeSignature = typeSignature;
    }
    
    private void setPrimitivType(Object obj, Object value, char nativSignature) 
        throws IllegalArgumentException, IllegalAccessException {
        
        if (this.typeSignature.charAt(0) == nativSignature) {
            setTypeSafe(obj, value);
        } else throw new IllegalArgumentException();
        
    }
    
    private void setTypeSafe(Object obj, Object value) {
        nativeSet(obj, value);
    }

    public void set(Object obj, Object value) throws IllegalArgumentException,
            IllegalAccessException {
        if (GWTRUtil.getClass(value) != getType()) {        
            throw new IllegalArgumentException();
        }
        try {
            setTypeSafe(obj, value);
        } catch (Exception ex) {
            throw new IllegalAccessException();
        }
    }

    public Object get(Object obj) throws IllegalArgumentException,
            IllegalAccessException {
        return nativeGet(obj);
    }

    public Class getDeclaringClass() {
        return declaringClass;
    }

    public String getName() {
        return name;
    }

    public int getModifiers() {
        return modifiers;
    }

    private String getTypeSignature() {
        return typeSignature;
    }

    public Class getType() {
        return GWTRUtil.toType(getTypeSignature());
    }

    public boolean getBoolean(Object obj) throws IllegalArgumentException,
            IllegalAccessException {
        return ((Boolean) get(obj)).booleanValue();
    }

    public byte getByte(Object obj) throws IllegalArgumentException,
            IllegalAccessException {
        return ((Byte) get(obj)).byteValue();
    }

    public char getChar(Object obj) throws IllegalArgumentException,
            IllegalAccessException {
        return ((Character) get(obj)).charValue();
    }

    public double getDouble(Object obj) throws IllegalArgumentException,
            IllegalAccessException {
        return ((Double) get(obj)).doubleValue();
    }

    public float getFloat(Object obj) throws IllegalArgumentException,
            IllegalAccessException {
        return ((Float) get(obj)).floatValue();
    }

    public int getInt(Object obj) throws IllegalArgumentException,
            IllegalAccessException {
        return ((Integer) get(obj)).intValue();
    }

    public long getLong(Object obj) throws IllegalArgumentException,
            IllegalAccessException {
        return ((Long) get(obj)).longValue();
    }

    public short getShort(Object obj) throws IllegalArgumentException,
            IllegalAccessException {
        return ((Short) get(obj)).shortValue();
    }

    public void set(Object obj, boolean value) throws IllegalArgumentException,
            IllegalAccessException {
        setPrimitivType(obj, new Boolean(value),NativSignatures.BOOLEAN_TYPE);
    }

    public void set(Object obj, byte value) throws IllegalArgumentException,
            IllegalAccessException {
        setPrimitivType(obj, new Byte(value),NativSignatures.BYTE_TYPE);
    }

    public void set(Object obj, char value) throws IllegalArgumentException,
            IllegalAccessException {
        setPrimitivType(obj, new Character(value),NativSignatures.CHAR_TYPE);
    }

    public void set(Object obj, double value) throws IllegalArgumentException,
            IllegalAccessException {
        setPrimitivType(obj, new Double(value),NativSignatures.DOUBLE_TYPE);
    }

    public void set(Object obj, float value) throws IllegalArgumentException,
            IllegalAccessException {
        setPrimitivType(obj, new Float(value),NativSignatures.FLOAT_TYPE);
    }

    public void set(Object obj, int value) throws IllegalArgumentException,
            IllegalAccessException {
        setPrimitivType(obj, new Integer(value),NativSignatures.INT_TYPE);
    }

    public void set(Object obj, long value) throws IllegalArgumentException,
            IllegalAccessException {
        setPrimitivType(obj, new Long(value),NativSignatures.LONG_TYPE);
    }

    public void set(Object obj, short value) throws IllegalArgumentException,
            IllegalAccessException {
        
        setPrimitivType(obj, new Short(value),NativSignatures.SHORT_TYPE);
        
    }

}
