/* pl.rmalinowski.gwtreflect.client.reflect.GWTClass.java 
 Copyright (C) 1998, 1999, 2001, 2005  Free Software Foundation, Inc.

 This file is part of GNU Classpath.

 GNU Classpath is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 GNU Classpath is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
 Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 02110-1301 USA.

 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
 conditions of the GNU General Public License cover the whole
 combination.

 As a special exception, the copyright holders of this library give you
 permission to link this library with independent modules to produce an
 executable, regardless of the license terms of these independent
 modules, and to copy and distribute the resulting executable under
 terms of your choice, provided that you also meet, for each linked
 independent module, the terms and conditions of the license of that
 module.  An independent module is a module which is not derived from
 or based on this library.  If you modify this library, you may extend
 this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */

package pl.rmalinowski.gwtreflect.client.reflect;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import pl.rmalinowski.gwtreflect.client.utils.NativSignatures;

import com.google.gwt.core.client.GWT;

public class GWTRUtil {
    private static boolean isOutOfScopeClassEmulatorEnable = true;

    /**
     * @gwt.typeArgs <java.lang.String,pl.rmalinowski.gwtreflect.client.reflect.Class>
     */
    private final static Map classes = new HashMap();

    /*
     * { signatures.put(void.class, "V"); signatures.put(boolean.class, "Z");
     * signatures.put(byte.class, "B"); signatures.put(char.class, "C");
     * signatures.put(short.class, "S"); signatures.put(int.class, "I");
     * signatures.put(long.class, "J"); signatures.put(float.class, "F");
     * signatures.put(double.class, "D");
     */
    
    public static Boolean toObject(boolean v) {
        return new Boolean(v);
    }    
    public static boolean booleanValue(Object v) {
        return ((Boolean)v).booleanValue();
    }    
    public static Byte toObject(byte v) {
        return new Byte(v);
    }
    public static byte byteValue(Object v) {
        return ((Byte)v).byteValue();
    }
    public static Character toObject(char v) {
        return new Character(v);
    }
    public static char charValue(Object v) {
        return ((Character)v).charValue();
    }
    public static Short toObject(short v) {
        return new Short(v);
    }
    public static short shortValue(Object v) {
        return ((Short)v).shortValue();
    }
    public static Integer toObject(int v) {
        return new Integer(v);
    }   
    public static int intValue(Object v) {
        return ((Integer)v).intValue();
    }
    public static Long toObject(long v) {
        return new Long(v);
    }
    public static long longValue(Object v) {
        return ((Long)v).longValue();
    }
    public static Float toObject(float v) {
        return new Float(v);
    }
    public static float floatValue(Object v) {
        return ((Float)v).floatValue();
    }
    public static Double toObject(double v) {
        return new Double(v);
    } 
    public static double doubleValue(Object v) {
        return ((Double)v).doubleValue();
    }

    public static Class getClass(String className)
            throws GWTRClassNotFoundException {
        return toType(toSignature(className));
        // return (Class) GWTClass.classes.get(className);
    }

    public static Class getClass(Object object)
            throws GWTRClassNotFoundException {
        return toType(toSignature(GWT.getTypeName(object)));
        // return (Class) GWTClass.classes.get(className);
    }

    public static Class getClass(java.lang.Class clazz)
            throws GWTRClassNotFoundException {
        return toType(toSignature(getName(clazz)));
        // return (Class) GWTClass.classes.get(clazz.toString().substring(6));
    }

    static Class addClass(String signature, Class type) {
        return (Class) classes.put(signature, type);
    }

    /**
     * Search for instance of Class in clasical way (toType), but if not found
     * returning ClassEmulator, which simulate implementation for class that are
     * out of the scope of GWT Reflection.
     * 
     * @param signature
     * @return Class or ClassEmulator
     */
    /*
     * public static Class toTypeOrTypeEmulator(String signature) { try { return
     * toType(signature); } catch (GWTRClassNotFoundException ex) { return new
     * ClassEmulator(signature); } }
     */

    public static Class toType(String signature)
            throws GWTRClassNotFoundException {
        Class clazz = null;
        if ((clazz = (Class) classes.get(signature)) == null) {
            if (isOutOfScopeClassEmulatorEnable()) {
                classes.put(signature, clazz = new ClassEmulator(signature));
                return clazz;
            } else {
                throw new GWTRClassNotFoundException(signature);
            }
        } else {
            return clazz;
        }

        /*
         * Iterator iter = classes.values().iterator(); while (iter.hasNext()) {
         * Class gwtClass = (Class) iter.next(); if
         * (gwtClass.getSignature().equals(signature)) return gwtClass; } return
         * new ProxySignatureClass(signature);
         */
    }

    public static Class[] toTypes(String[] signatures)
            throws GWTRClassNotFoundException {

        int length = signatures.length;
        Class[] classTypes = new Class[length];
        for (int i = 0; i < length; i++) {
            classTypes[i] = GWTRUtil.toType(signatures[i]);
        }
        return classTypes;
    }

    public static String[] toSignatures(Class[] types) {
        int length = types.length;
        String[] classSignatures = new String[length];
        for (int i = 0; i < length; i++) {
            classSignatures[i] = types[i].getSignature();
        }
        return classSignatures;
    }

    public static String toSignatureString(String[] signatures) {
        String s = "";
        for (int i = 0; i < signatures.length; i++) {
            s += signatures[i];
        }
        return s;
    }

    public static String toSignatureString(Class[] types) {
        String s = "";
        for (int i = 0; i < types.length; i++) {
            s += types[i].getSignature();
        }
        return s;
    }

    /**
     * toString return: (isInterface() ? "interface " : (isPrimitive() ? "" :
     * "class ")) + getName();
     * 
     * @param clazz
     * @return
     */
    public static String getName(java.lang.Class clazz) {
        String className = clazz.toString();
        if (className.length() > 5
                && className.substring(0, 6).equals("class ")) {
            return className.substring(6);
        } else if (className.length() > 9
                && className.substring(0, 10).equals("interface ")) {
            return className.substring(10);
        } else {
            return className;
        }

        // return (clazz.isInterface() ? clazz.toString().substring(10) : (clazz
        // .isPrimitive() ? clazz.toString() : clazz.toString().substring(
        // 6)));

        // return clazz.toString().substring(6);
    }

    
    public static String toClassName(String signature) {
        char firstLetter = signature.charAt(0);

        switch (firstLetter) {
        case NativSignatures.VOID_TYPE:
            return "void";
        case NativSignatures.BOOLEAN_TYPE:
            return "boolean";
        case NativSignatures.BYTE_TYPE:
            return "byte";
        case NativSignatures.CHAR_TYPE:
            return "char";
        case NativSignatures.SHORT_TYPE:
            return "short";
        case NativSignatures.INT_TYPE:
            return "int";
        case NativSignatures.LONG_TYPE:
            return "long";
        case NativSignatures.FLOAT_TYPE:
            return "float";
        case NativSignatures.DOUBLE_TYPE:
            return "double";
        case NativSignatures.ARRAY_TYPE:
            return "array";
        case NativSignatures.OBJECT_TYPE:
            return signature.replace('/', '.').substring(1,
                    signature.length() - 1);
        default:
            return "null";
        }

    }

    public static String toSignature(String className) {
        // becouse HelloWorld.class.toString() == "class
        // org.me.packagename.HelloWorld"
        // and becouse something like this should work
        // Class.forName(HelloWorld.class.toString())
        // we check is 'class ' prefix exist
        // if (className.length() > 5
        // && className.substring(0, 6).equals("class ")) {
        // className = className.substring(6); // and if, we cut it.
        // }
        if (className.equals("void")) {
            return String.valueOf(NativSignatures.VOID_TYPE);
        } else if (className.equals("boolean")) {
            return String.valueOf(NativSignatures.BOOLEAN_TYPE);
        } else if (className.equals("byte")) {
            return  String.valueOf(NativSignatures.BYTE_TYPE);
        } else if (className.equals("char")) {
            return  String.valueOf(NativSignatures.CHAR_TYPE);
        } else if (className.equals("short")) {
            return  String.valueOf(NativSignatures.SHORT_TYPE);
        } else if (className.equals("int")) {
            return  String.valueOf(NativSignatures.INT_TYPE);
        } else if (className.equals("long")) {
            return  String.valueOf(NativSignatures.LONG_TYPE);
        } else if (className.equals("float")) {
            return  String.valueOf(NativSignatures.FLOAT_TYPE);
        } else if (className.equals("double")) {
            return  String.valueOf(NativSignatures.DOUBLE_TYPE);
        } else if (className.equals("array")) {
            return  String.valueOf(NativSignatures.ARRAY_TYPE);
        }
        return  String.valueOf(NativSignatures.OBJECT_TYPE) + className.replace('.', '/') + ";";
    }

    public static String toSignatureString(java.lang.Class[] types) {
        String s = "";
        for (int i = 0; i < types.length; i++) {
            s += GWTRUtil.toSignature(GWTRUtil.getName(types[i]));
        }
        return s;
    }

    public static String toMethodSignaturedName(String methodName,
            String signaturesString) {
        return methodName + "(" + signaturesString + ")";
    }

    public static boolean isOutOfScopeClassEmulatorEnable() {
        return isOutOfScopeClassEmulatorEnable;
    }

    public static void setOutOfScopeClassEmulatorEnable(
            boolean isOutOfScopeClassEmulatorEnable) {
        GWTRUtil.isOutOfScopeClassEmulatorEnable = isOutOfScopeClassEmulatorEnable;
    }

}
